// Copyright (c) 2014 Tel-Aviv University (Israel).
// All rights reserved.
//
// This file is part of ACG (www.acg.cs.tau.ac.il).
// You can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Author(s): Michal Kleinbort <balasmic@post.tau.ac.il>
//            Oren Salzman <orenzalz@post.tau.ac.il >          

#ifndef POINT_D_H__RTG
#define POINT_D_H__RTG

#include <iostream>

namespace RTG
{

    /// A class used for defining a d-dimensional point
    /*!
        A single transformed grid that defines a partition 
        of the data into cells of a given size.
        Nearby points that lie in the same cell are reported.

        @par It is possible to use different classes for d-dimensional points.
        However, these classes must implement the following methods:
        -  double& operator[](unsigned int idx)
        -  const double& operator[](unsigned int idx) const
    */
    class Point_d_
    {
     public:

        /// \brief Constructor given an input iterator
	    template <class InputIterator>
	    Point_d_(unsigned int dim, InputIterator first, InputIterator last)
            :m_dim(dim)
        {
            m_coords = new double[m_dim];
            InputIterator curr = first;
            int i(0);

            while (curr != last)
            {
                m_coords[i] = *curr;
                i++;
                curr++;
            }
	    }

        /// \brief Copy constructor
        Point_d_(const Point_d_& p)
            :m_dim(p.dimension())
        {
            m_coords = new double[m_dim];
      
            for (unsigned int i=0; i < m_dim; ++i)
                m_coords[i] = p[i];
	    }

        /// \brief Destructor
	    ~Point_d_()
	    {
		    delete [] m_coords;
	    }

      
        /// \brief retrieves the i-th coordinate of the d-dimensional point
        /// This method must be implmeneted 
	    double& operator[](unsigned int idx)
	    {
		    assert (idx < m_dim);
		    return m_coords[idx];
	    }


        /// \brief retrieves the i-th coordinate of the d-dimensional point
        /// This method must be implmeneted
	    const double& operator[](unsigned int idx) const
	    {
		    assert(idx < m_dim);
		    return m_coords[idx];
	    }

        
        /// \brief returns the dimension of the point
        unsigned int dimension() const
	    {
		    return m_dim;
	    }

	    /// \brief operator<< for outputing the point
        friend std::ostream &operator<<( std::ostream &output, const Point_d_ & p )
        { 
            output << "(";
		    for (unsigned int i(0); i < p.dimension()-1; ++i)
			    output << p.m_coords[i] << ", ";
		    output <<  p.m_coords[p.dimension()-1] << ")";
            return output;            
        }

     private:

        /// \brief an array of the coordinates
	    double* m_coords;
	     /// \brief the dimension of the point
        unsigned int m_dim;

    }; //Point_d_

} //namespace RTG

#endif //POINT_D_H__RTG